 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33FG0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED:     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.  
  *
  ******************************************************************************************************* 
  */ 

#ifndef __LIN_H__
#define __LIN_H__
#include "main.h"

/* 功能支持 */
#define SLEEP_SUPPORT       0   /* Lin休眠时是否支持sleep 0-不支持 1-支持 */
#define AUTOBAUD            1   /* 0-固定波特率 1-自适应波特率 */

/* 执行时间优化相关宏定义 */
#define LIN_UART_IRQ_PRI        1
#define LIN_TIMER_IRQ_PRI       2
#define LIN_GPIO_IRQ_PRI        2

/* LIN 相关宏定义 */
#define __SYSTEM_CLOCK      (32000000)
#define MAIN_CLOCK          __SYSTEM_CLOCK                  /* 主频, 单位Hz */
#define TIMER_1MS_LOAD      (MAIN_CLOCK/1000)               /* 定时器1ms, Load值 */ 
#define TIMER_1US_LOAD      (MAIN_CLOCK/1000000)            /* 定时器1us, Load值 */
#define TIME_BASE_PERIOD    500                             /* 接收超时定时器, 单位us */
#define TIMER_LOAD_VALUE    TIMER_1US_LOAD * TIME_BASE_PERIOD
#define LIN_IDLE_TIMEOUT    10000                           /* TIME_BASE_PERIOD为单位,休眠周期,此时间段内无通信则进入sleep,有通信则重置超时 */

/* 校验类型 */
#define CHECKSUM_TYPE_STANDARD   0
#define CHECKSUM_TYPE_ENHANCED   1
#define CHECKSUM_TYPE            CHECKSUM_TYPE_STANDARD     /* 0-标准型 1-增强型 */

typedef enum {
    LIN_CMD0 = 0x3C,
    LIN_CMD1 = 0x3D,  
    LIN_CMD2 = 0x38,  
    /* 此处增加自定义ID */
}lin_cmd_t;

typedef enum {
    TRANSMIT,   /* 主机发送 */
    RECEIVE     /* 主机接收 */
}lin_packet_type_t;

typedef struct {
    uint8_t cmd;           /* 参考LIN_CMD0 */
    lin_packet_type_t type;/* 命令类型 */
    uint8_t length;
    uint8_t* data;
    uint8_t pid;           /* 转换cmd为pid, 初始化时进行 */
    volatile uint8_t FullFlg;   /* 有数据标记 */
}lin_cmd_packet_t;

#define LIN_BREAK_BIT       13                                          /* 同步间隔, 13bit显性电平 */
#define LIN_BREAK_BYTE      0x00
#define LIN_SYN_BYTE        0x55

/* lin波特率和字节间间隔 */
#define LIN_BAUDRATE                    19200                           /* 波特率, 上限20K */
#define LIN_RECV_BYTE_TIME_SPACE        12                              /* 接收字节间间隔, 最大支持255bit */

#define LIN_SYN_BYTE_TIME_SPACE         50                              /* 收到间隔段之后 */
#define LIN_BIT_TIMER_LOAD  (MAIN_CLOCK / LIN_BAUDRATE)                 /* 比特定时器Load值, TIME时钟源24MHz */
#define RESPONSE_SPACE      (50 * LIN_BIT_TIMER_LOAD)                   /* 应答间隔, 5字节长度, 每字节10bit */

#define LIN_UART                        UART0
#define LIN_UART_RX_GPIO                GPIOA
#define LIN_UART_RX_PIN                 FL_GPIO_PIN_2
#define LIN_UART_RX_REMAP_FUNCTION      FL_GPIO_PINREMAP_FUNCTON1
#define LIN_UART_TX_GPIO                GPIOA
#define LIN_UART_TX_PIN                 FL_GPIO_PIN_3
#define LIN_UART_TX_REMAP_FUNCTION      FL_GPIO_PINREMAP_FUNCTON1
#define LIN_UART_RX_PIN_FCR_MASK        0x0C000000                      /* (3 << (LIN_UART_RX_PIN_IDX << 1)) */
#define LIN_UART_RX_PIN_FCR_MASK_INV    0xF3FFFFFF                      /* 取反 */
#define LIN_UART_RX_PIN_FCR_INPUT       0                               /* (0 << (LIN_UART_RX_PIN_IDX << 1)) */
#define LIN_UART_RX_PIN_FCR_DIGIT       0x08000000                      /* (2 << (LIN_UART_RX_PIN_IDX << 1)) */
#define LIN_RX_EXTI_INPUT_GROUP         FL_GPIO_EXTI_INPUT_GROUP0
#define LIN_RX_EXTI_LINE                FL_GPIO_PIN_13
#define LIN_RX_EXTI_LINE_PIN            FL_GPIO_PIN_13
#define LIN_RX_EXTI_LINE_INDEX          6
#define LIN_RX_EXTI_EDS_INDEX           12
#define LIN_UART_IRQHandler             UART0_IRQHandler
#define LIN_UART_IRQType                UART0_IRQn

////#define LIN_UART_CAPTURE_GPIO               GPIOA
////#define LIN_UART_CAPTURE_PIN                FL_GPIO_PIN_10
////#define LIN_UART_CAPTURE_PIN_INV            0xFFFFFBFF
////#define LIN_UART_CAPTURE_PIN_IDX            10
////#define LIN_UART_CAPTURE_PIN_FCR_MASK       0x00300000                  /* (3 << (LIN_UART_CAPTURE_PIN_IDX << 1)) 便于优化执行时间 */
////#define LIN_UART_CAPTURE_PIN_FCR_MASK_INV   0xFFCFFFFF                  /* 取反，便于优化执行时间 */
////#define LIN_UART_CAPTURE_PIN_FCR_OUTPUT     0x00100000                  /* (0 << (LIN_UART_CAPTURE_PIN_IDX << 1)) */
////#define LIN_UART_CAPTURE_PIN_FCR_DIGIT      0x00200000                  /* (2 << (LIN_UART_CAPTURE_PIN_IDX << 1)) */

#define LIN_SLP_GPIO        GPIOA
#define LIN_SLP_PIN         FL_GPIO_PIN_11
#define LIN_SLP_ENABLE()    FL_GPIO_ResetOutputPin(LIN_SLP_GPIO, LIN_SLP_PIN)
#define LIN_SLP_DISABLE()   FL_GPIO_SetOutputPin(LIN_SLP_GPIO, LIN_SLP_PIN)

/* 用户处理 */
#define LIN_CMD0_LENGTH  8
#define LIN_CMD1_LENGTH  8
#define LIN_CMD2_LENGTH  8

extern uint8_t LIN_CMD0_Data[8];
extern uint8_t LIN_CMD1_Data[8];
extern uint8_t LIN_CMD2_Data[8];
extern lin_cmd_packet_t scheduleTable[];
#define TABLE_SIZE  (sizeof(scheduleTable)/sizeof(lin_cmd_packet_t))

void lin_init(void);

#if (SLEEP_SUPPORT == 1)
void lin_sleep_init(void);

void lin_awake_init(void);

uint8_t lin_is_sleep(void);
#endif

extern int32_t LIN_TxMsgFill(uint8_t *data, uint32_t lenth);

#endif
