#ifndef __PLL_H__
#define __PLL_H__

#include "fm33lf0xxa_fl.h"
#include <stdio.h>


#define LED_RED_GPIO         GPIOC
#define LED_RED_PIN          FL_GPIO_PIN_1


/*----------LED_RED-----------------------------------------------------------*/
#define LED_RED_ON()     do{FL_GPIO_ResetOutputPin(LED_RED_GPIO, LED_RED_PIN);}while(0)
#define LED_RED_OFF()    do{FL_GPIO_SetOutputPin(LED_RED_GPIO, LED_RED_PIN);}while(0)
#define LED_RED_TOG()    do{FL_GPIO_ToggleOutputPin(LED_RED_GPIO, LED_RED_PIN);}while(0)


typedef enum
{
    SYSCLK_NORMAL = 0x00000000,
    SYSCLK_ERROR  = 0x55AAAA55,
}SYSCLK_STATE;

/**
  * @brief FL CMU RCHF Init Sturcture definition
  */
typedef struct
{
    /* RCHF频率 */
    uint32_t Frequency;
}FL_CMU_RCHF_InitTypeDef;

/**
  * @brief FL CMU XTHF Init Sturcture definition
  */
typedef struct
{
    /* XTHF频率 */
    uint32_t Frequency;
    /* XTHF启动等待时间 */
    uint32_t StartWaitTime;
    /* XTHF Bypss使能或禁止 */
    uint32_t Bypass;
    /* XTHF振荡强度 */
    uint32_t DriverStrength;
}FL_CMU_XTHF_InitTypeDef;

/**
  * @brief FL CMU PLL Init Sturcture definition
  */
typedef struct
{
    /* PLL倍频比 */
    uint32_t Multiplier;
    /* PLL参考时钟预分频 */
    uint32_t Prescaler;
    /* PLL时钟源输入选择 */
    uint32_t ClockSource;
}FL_CMU_PLL_InitTypeDef;

extern volatile uint32_t SYSCLKSELErrorState;
extern volatile SYSCLK_STATE SYSCLKState;

extern const FL_CMU_RCHF_InitTypeDef CMU_RCHF_Init_Data;
extern const FL_CMU_XTHF_InitTypeDef CMU_XTHF_Init_Data;
extern const FL_CMU_PLL_InitTypeDef CMU_PLL_Init_Data;
extern const FL_CMU_PLL_InitTypeDef CMU_PLL_Init_Data1;

extern uint32_t SwtichToSenondarySystemClock(const FL_CMU_RCHF_InitTypeDef *rchf, const FL_CMU_PLL_InitTypeDef *pll);
extern void SwtichToMainSystemClock(const FL_CMU_RCHF_InitTypeDef *rchf,
                                    const FL_CMU_XTHF_InitTypeDef *xthf,
                                    const FL_CMU_PLL_InitTypeDef  *pll);

void RCC_PLL_ConfigDomain_SYS(uint32_t Source, uint32_t PLL_R, uint32_t PLL_DB);

void RCHFInit(uint32_t RCHFclock);
uint32_t SelRCHFToPLL(uint32_t rchf, uint32_t PLLclock);

void XTHFInit(const FL_CMU_XTHF_InitTypeDef *init);
extern uint32_t SelXTHFToPLL(const FL_CMU_XTHF_InitTypeDef *xthfInit, 
                      const FL_CMU_PLL_InitTypeDef   *pllInit);

#endif
