 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33LF0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED:     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.  
  *
  ******************************************************************************************************* 
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __BOOTLOADER_H
#define __BOOTLOADER_H
/* Includes ------------------------------------------------------------------*/
#include "main.h"


/* Private typedef -----------------------------------------------------------*/
#define BOOT_RXMSGBUFF    4 
#define BOOT_COMM_CHL     3 

typedef struct
{
    uint8_t length;  /* data length */
    uint8_t Data[8]; /*!< Contains the data to be received. It ranges from 0 to 
                        0xFF. */
}BootRxMsg;


typedef struct
{
    volatile uint32_t MsgIndex;
    BootRxMsg RxMessage[BOOT_RXMSGBUFF];
}BootRxMsgBuff;


typedef struct
{
    void (*AppExeFlagCheck)(void);   /* APP执行标志检查  */
    void (*Configuration)(void);     /* 初始化  */
    int32_t(*RxMsgFill)(uint8_t *, uint32_t, uint32_t);     /* 接收信息填充 */
    void (*Process)(int32_t(*const TxMsgFill[BOOT_COMM_CHL])(uint8_t *, uint32_t));  /* Boot执行命令 */
}BootObject;


/* Private define ------------------------------------------------------------*/  
/** 
  * @brief  FLASH Status  
  */
typedef enum
{ 
  FLASH_BUSY = 1,
  FLASH_ERROR_PG,
  FLASH_ERROR_WRP,
  FLASH_COMPLETE,
  FLASH_TIMEOUT
    
}FLASH_Status;


/* Private macro -------------------------------------------------------------*/
#define ENCRYPT 1       /* 固件传输是否加密，若将此项配置为1，那么需要在app工程中勾选“After Build/Rebuild下的Run #1和Run #2” */
#define APP_START_ADDR             ((uint32_t)0x000A000)/* APP程序起始地址 */
#define APP_EXE_FLAG_ADDR          ((uint32_t)0x0007000)/* APP程序运行标志存储地址，该标志在APP程序中写入和擦除 */
/* 固件类型值定义 */
#define FW_TYPE_BOOT     0x55
#define FW_TYPE_APP      0xAA
/* 定义当前固件类型为BOOT */
#define FW_TYPE         FW_TYPE_BOOT
/* 定义数据收发的帧ID,必须跟上位机软件配置一致，否则无法正常工作 */
/* 定义数据收发帧ID类型，0-标准帧，1-扩展帧 */
#define MSG_ID_TYPE         0
#if (MSG_ID_TYPE == 0) /* 标准帧 */
    /* 对于CAN总线，数据收发ID可以定义为一个ID, 也可以定义为不同的ID */
    #define MSG_RECEIVE_ID      0x3C
    #define MSG_SEND_ID         0x3D
#else /* 扩展帧 */
    /* 对于CAN总线，数据收发ID可以定义为一个ID, 也可以定义为不同的ID */
    #define MSG_RECEIVE_ID      0x12345
    #define MSG_SEND_ID         0x12346
#endif
/* 定义广播节点的地址 */
#define NAD_BROADCAST           0x7F
/* BOOT命令定义 */
#define CMD_GET_FW_INFO         0x80
#define CMD_ENTER_BOOT          0xC1
#define CMD_ERASE_APP           0x42
#define CMD_SET_ADDR_OFFSET     0x03
#define CMD_TRAN_DATA           0xC4
#define CMD_WRITE_DATA          0x85
#define CMD_ENTER_APP           0x06
/* BOOT错误定义 */
#define BOOT_ERR_SUCCESS        0   /* 没有错误 */
#define BOOT_ERR_ERASE          1   /* 固件擦除错误 */
#define BOOT_ERR_ERASE_IN_APP   2   /* 当前模式为app，不能擦除固件 */
#define BOOT_ERR_WRITE_OUTRANGE 3   /* 当前地址超出了正常地址范围 */
#define BOOT_ERR_WRITE_IN_APP   4   /* 当前模式不能写入固件数据 */
#define BOOT_ERR_WRITE          5   /* 数据写入程序存储器出错 */
#define BOOT_ERR_WRITE_OUT_ADDR 6   /* 数据长度超出了地址范围 */
#define BOOT_ERR_TRAN_CRC       7   /* 数据传输CRC校验出错 */
#define BOOT_ERR_WRITE_CRC      8   /* 数据写入芯片CRC校验出错 */
/* Private variables ---------------------------------------------------------*/
extern const BootObject BOOT_Object;


/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/



#endif

/*********************************END OF FILE**********************************/

