 /**    
  *******************************************************************************************************    
  * @attention    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33LF0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED:     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.   
  *
  ******************************************************************************************************* 
  */    
#include "lptim16.h"
#include "pll.h"

/* 10ms 周期标志 */
volatile uint32_t Period10msFlag = 0;
/**
  * @brief  中断服务函数
  * @param  void
  * @retval void
  */
void LPTIM16_IRQHandler(void)
{
    /* 定时器溢出中断 */
    if(FL_LPTIM16_IsActiveFlag_Update(LPTIM16) && \
       FL_LPTIM16_IsEnabledIT_Update(LPTIM16))
    {
        /* 清除LPTIM16定时器溢出中断标志 */
        FL_LPTIM16_ClearFlag_Update(LPTIM16);
        Period10msFlag = 1;
    }
}


/**
  * @brief  LPTIM16初始化
  * @param  void
  * @retval void
  */
void LPTIM16_Init(void)
{
    FL_LPTIM16_InitTypeDef      TimInit;
    FL_NVIC_ConfigTypeDef       InterruptConfigStruct;
  
    /*-------------------定时器时基配置------------------- */
    TimInit.clockSource          = FL_CMU_LPTIM16_CLK_SOURCE_APBCLK;    /* 工作时钟源配置：    APBCLK */
    TimInit.mode                 = FL_LPTIM16_OPERATION_MODE_NORMAL;    /* 工作模式配置：      NORMAL */
    TimInit.prescalerClockSource = FL_LPTIM16_CLK_SOURCE_INTERNAL;      /* 预分频输入配置：    INTERNAL */
    TimInit.prescaler            = FL_LPTIM16_PSC_DIV64;                /* 预分频值配置：      64分频 */
    TimInit.autoReload           = 10000 - 1;                           /* 自动重装载值配置：  10000 */
    TimInit.encoderMode          = FL_LPTIM16_ENCODER_MODE_DISABLE;     /* 编码器模式使能配置：禁止 */
    TimInit.onePulseMode         = FL_LPTIM16_ONE_PULSE_MODE_CONTINUOUS;/* 单次计数模式配置：  禁止 */
    TimInit.triggerEdge          = FL_LPTIM16_ETR_TRIGGER_EDGE_RISING;  /* ETR触发沿配置：     上升沿 */
    TimInit.countEdge            = FL_LPTIM16_ETR_COUNT_EDGE_RISING;    /* ETR计数边沿配置：   上升沿 */
    FL_LPTIM16_Init(LPTIM16, &TimInit);
  
    /*-------------------定时器中断配置------------------- */
    /* 清除标志 */
    FL_LPTIM16_ClearFlag_Update(LPTIM16);

    /* 中断使能 */
    FL_LPTIM16_EnableIT_Update(LPTIM16);
  
    /* 配置NVIC */
    InterruptConfigStruct.preemptPriority = 0x02;     
    FL_NVIC_Init(&InterruptConfigStruct, LPTIM16_IRQn);
  
    /* 使能LPTIM16 */
    FL_LPTIM16_Enable(LPTIM16);
}


/** 
  * @brief  测试IO初始化
  * @param  void
  * @retval void
  */
void TEST_IO_Init(void)
{
    FL_GPIO_InitTypeDef           GPIO_InitStruct;
    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_0;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_OPENDRAIN;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON1;

    FL_GPIO_Init(GPIOC, &GPIO_InitStruct);
}


