 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33LF0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED:     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.    
  *
  *******************************************************************************************************
  */    
  
#include "pll.h"
#define PLL_TIMEOUT      0xFFFFFFFFU
#define XTHF_TIMEOUT     0xFFFFFFFFU

/* XTHF寄存器参数检查 */
#define XTHF_REG_CHECK(xthf)  ((FL_CMU_XTHF_GetStartWaitTime()   == (xthf)->StartWaitTime) &&\
                               (FL_CMU_IsEnabled_XTHF_Bypass()    == (xthf)->Bypass) &&\
                               (FL_CMU_XTHF_ReadDriverStrength() == (xthf)->DriverStrength) &&\
                               (FL_CMU_XTHF_IsEnabled()          == FL_ENABLE))

/* 系统时钟选择错误变量 */
volatile uint32_t SYSCLKSELErrorState = 0;
/* 系统时钟状态变量 */
volatile SYSCLK_STATE SYSCLKState = SYSCLK_NORMAL;

/* RCHF初始化参数 */
const FL_CMU_RCHF_InitTypeDef CMU_RCHF_Init_Data = {
    /* RCHF频率必须≤24MHz, 否则需要考虑Code Wait */
    /* 当系统时钟使用PLL时，RCHF频率和XTHF频率可以不一致，因为可以通过PLL得到相同目标频率，推荐固定为8MHz */
    FL_CMU_RCHF_FREQUENCY_8MHZ
};

/* XTHF初始化参数 */
const FL_CMU_XTHF_InitTypeDef CMU_XTHF_Init_Data = {
    /* XTHF频率值，必须与外部实际晶体匹配 */
    8000000,
    /* XTHF启动等待时间 */
    FL_CMU_XTHF_START_WAIT_CYCLE_16384,
    /* XTHF Bypss使能或禁止 */
    FL_DISABLE,
    /* XTHF振荡强度 */
    0xA,
};

/* PLL初始化参数 */
const FL_CMU_PLL_InitTypeDef CMU_PLL_Init_Data = {
    /* PLL将1M时钟倍频至48MHz */
    32,  
    /* PLL将参考时钟预分频至1MHz*/
    FL_CMU_PLL_PSC_DIV8,
    /* PLL时钟源输入选择 */
    FL_CMU_PLL_CLK_SOURCE_XTHF,
};

/* PLL初始化参数 */
const FL_CMU_PLL_InitTypeDef CMU_PLL_Init_Data1 = {
    /* PLL将1M时钟倍频至48MHz */
    64,  
    /* PLL将参考时钟预分频至1MHz*/
    FL_CMU_PLL_PSC_DIV8,
    /* PLL时钟源输入选择 */
    FL_CMU_PLL_CLK_SOURCE_XTHF,
};

void FDET_IRQHandler(void)
{
    uint32_t SYSCLKSELErrorIT = 0;
    uint32_t SYSCLKSELErrorFlag = 0;
    uint32_t HFDET_IT = 0;
    uint32_t HFDET_IF = 0;

    SYSCLKSELErrorIT = FL_CMU_IsEnabledIT_SYSCLKSelectError();
    SYSCLKSELErrorFlag = FL_CMU_IsActiveFlag_SysclkSecleError();

    HFDET_IT = FL_CMU_IsEnabledIT_XTHFFail();
    HFDET_IF = FL_CMU_IsActiveFlag_XTHFFail();
    
    /*停振检测中断处理*/
    if ((HFDET_IT == 0x01UL)&& (HFDET_IF == 0x01UL))
    {
        /*切换到次系统时钟*/
        SwtichToSenondarySystemClock(&CMU_RCHF_Init_Data, &CMU_PLL_Init_Data);
    }
    
    /*时钟选择中断处理*/
    if((SYSCLKSELErrorIT == 0x01UL)&& (SYSCLKSELErrorFlag == 0x01UL))
    {
        FL_CMU_ClearFlag_SysclkSecleError();
        SYSCLKSELErrorState = 1;
    }
}


/**
  * @brief  切换到次系统时钟
  * @param  const FL_CMU_RCHF_InitTypeDef *rchf
  *             RCHF参数
  *         const FL_CMU_PLL_InitTypeDef *pll
  *             PLL参数
  * @retval None
  */
uint32_t SwtichToSenondarySystemClock(const FL_CMU_RCHF_InitTypeDef *rchf,
                                  const FL_CMU_PLL_InitTypeDef  *pll)
{
    FL_NVIC_ConfigTypeDef    InterruptConfigStruct;

    uint32_t counter = 0;
    uint32_t readystatus = 0;
    uint32_t PLLdiv = FL_CMU_PLL_PSC_DIV8;

    if(pll->Multiplier > 72)
    {
        /* 超出PLL倍频最高范围 */
        return 1;
    }
    else
    {
        RCHFInit(rchf->Frequency);
        /* 清除时钟选择错误标志 */
        FL_CMU_ClearFlag_SysclkSecleError();
        /* 使能时钟选择错误中断 */
        FL_CMU_EnableIT_SYSCLKSelectError();

        InterruptConfigStruct.preemptPriority = 0x00;
        /* 系统时钟选择错误中断 */
        FL_NVIC_Init(&InterruptConfigStruct, FDET_IRQn);    

        switch(rchf->Frequency)
        {
            case FL_CMU_RCHF_FREQUENCY_16MHZ:
                PLLdiv = FL_CMU_PLL_PSC_DIV16;
                break;
            case FL_CMU_RCHF_FREQUENCY_24MHZ:
                PLLdiv = FL_CMU_PLL_PSC_DIV24;
                break;
            case FL_CMU_RCHF_FREQUENCY_32MHZ:
                PLLdiv = FL_CMU_PLL_PSC_DIV32;
                break;
            default:
                PLLdiv = FL_CMU_PLL_PSC_DIV8;
                break;
        }
        FL_FLASH_SetReadWait(FLASH,FL_FLASH_READ_WAIT_2CYCLE);

        RCC_PLL_ConfigDomain_SYS(FL_CMU_PLL_CLK_SOURCE_RCHF, PLLdiv, pll->Multiplier);

        FL_CMU_PLL_Enable();
        counter = 0x00000FFFU;
        while((0x0 == FL_CMU_IsActiveFlag_PLLReady()) && (--counter));
        do
        {
            readystatus = FL_CMU_IsActiveFlag_PLLReady();
            counter++;

            if((counter % 100) == 0)
            {
                FL_IWDT_ReloadCounter(IWDT); 
            }
        } while((readystatus != 0x1U) && (counter != PLL_TIMEOUT));

        FL_CMU_SetAHBPrescaler(FL_CMU_AHBCLK_PSC_DIV1);
        FL_CMU_SetAPBPrescaler(FL_CMU_APBCLK_PSC_DIV1);
        FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);

        if(pll->Multiplier <= 24)
        {
            /* Flash读等待周期配置 0 wait cycle */
            FL_FLASH_SetReadWait(FLASH,FL_FLASH_READ_WAIT_0CYCLE);
        }
        else  if(pll->Multiplier <= 48)  
        {
            /* Flash读等待周期配置 1 wait cycle */
            FL_FLASH_SetReadWait(FLASH,FL_FLASH_READ_WAIT_1CYCLE); 
        }
        else 
        {
            /* Flash读等待周期配置 2 wait cycle */
            FL_FLASH_SetReadWait(FLASH,FL_FLASH_READ_WAIT_2CYCLE);
        }
    }
    /* 清XTHF停振中断标识 */
    FL_CMU_ClearFlag_XTHFFail();
    /* 禁止高频检测报警中断 */
    FL_CMU_DisableIT_XTHFFail();
    
    FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);
    SYSCLKState = SYSCLK_ERROR;
    SystemCoreClockUpdate();
    
    FL_DelayMs(500);
    return 0;
}


/**
  * @brief  切换到主系统时钟
  * @param  const FL_CMU_RCHF_InitTypeDef *rchf,
  *              RCHF参数
  *         const FL_CMU_XTHF_InitTypeDef *xthf
  *              XTHF参数
  *         const FL_CMU_PLL_InitTypeDef *pll
  *              PLL参数
  * @retval None
  */
void SwtichToMainSystemClock(const FL_CMU_RCHF_InitTypeDef *rchf,
                             const FL_CMU_XTHF_InitTypeDef *xthf,
                             const FL_CMU_PLL_InitTypeDef  *pll)
{
    if ((SYSCLK_ERROR == SYSCLKState) &&\
        ((FL_CMU_SYSTEM_CLK_SOURCE_PLL != FL_CMU_GetSystemClockSource()) ||\
         (pll->ClockSource != FL_CMU_PLL_GetClockSource())))
    {
        /* 即使发生停振或失锁事件，XTHF寄存器参数不应发生变化 */
        if (XTHF_REG_CHECK(xthf) || XTHF_REG_CHECK(xthf))
        {
            volatile uint32_t Reday[3];  /* 避免编译器优化 */
            uint32_t counter, primask;
            
            /* 判断XTHF稳定标志是否建立，连续读三次 */
            Reday[0] = FL_CMU_XTHF_IsReady();
            Reday[1] = FL_CMU_XTHF_IsReady();
            Reday[2] = FL_CMU_XTHF_IsReady();
            if (Reday[0] && Reday[1] && Reday[2])
            {
                /* 清XTHF停振中断标识 */
                FL_CMU_ClearFlag_XTHFFail(); 
                if (0x0 == FL_CMU_IsActiveFlag_XTHFFail())
                {
                    /* 准备切换到主系统时钟相关工作 */
                    RCHFInit(rchf->Frequency);
                    /* 竞态保护 */
                    primask = __get_PRIMASK();
                    __disable_irq();
                    FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_RCHF);
                    FL_CMU_PLL_Disable();
                    RCC_PLL_ConfigDomain_SYS(pll->ClockSource, pll->Prescaler, pll->Multiplier);
                    FL_CMU_PLL_Enable();
                    /* 等待PLL建立，3个RCLP Clock，约91.55us */
                    counter = PLL_TIMEOUT;
                    do
                    {   /* 判断PLL锁住标志，连续读三次 */
                        Reday[0] = FL_CMU_IsActiveFlag_PLLReady();
                        Reday[1] = FL_CMU_IsActiveFlag_PLLReady();
                        Reday[2] = FL_CMU_IsActiveFlag_PLLReady();
                        if (Reday[0] && Reday[1] && Reday[2])
                        {
                            volatile uint32_t counter = 0;
                            /* 通过内核时钟计算100us延时的计数个数 */
                            uint32_t RCLP_Clock = (SystemCoreClock / 1000000) * 100;
                            /* 必须等待3个RCLP的Clock，确保初始化PLL过程中如产生PLL失锁标志时可得以清除 */
                            for (counter = 0; counter < RCLP_Clock; counter++)
                            {
                                __NOP();
                            }

                            if (FL_CMU_IsActiveFlag_XTHFFail()) break; 
                            SYSCLKState = SYSCLK_NORMAL;
                            FL_CMU_EnableIT_XTHFFail();
                            FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);
                            SystemCoreClockUpdate();
                            break;
                        }
                    }while((0x0 == FL_CMU_IsActiveFlag_XTHFFail()) && (--counter));
                    /* 未成功切换到主系统时钟，则恢复到次系统时钟 */
                    if (SYSCLK_NORMAL != SYSCLKState)
                    {
                        SwtichToSenondarySystemClock(rchf, pll);
                    }
                    /* 取消竞态保护 */
                    __set_PRIMASK(primask);
                }
            }
        }
        else
        {   
            RCHFInit(rchf->Frequency);
            FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_RCHF);
            FL_CMU_PLL_Disable();
            /* 禁止高频检测报警中断 */
            FL_CMU_DisableIT_XTHFFail();
            SYSCLKState = SYSCLK_NORMAL;
            (void)SelXTHFToPLL(&CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
        }
    }
}



/**
  * @brief
  * @rmtoll   PLLCR    OSEL    LL_RCC_GetPLLDigitalDomainOutput
  * @param    Source This parameter can be one of the following values:
  *           @arg @ref FL_RCC_PLL_CLK_SOURCE_RCHF
  *           @arg @ref FL_RCC_PLL_CLK_SOURCE_XTHF
  * @param    PLL_R 锁相环的参考时钟需要预分频至1M，再进行PLL倍频 This parameter can be one of the following values:
  *           @arg @ref FL_RCC_PLL_PSC_DIV1
  *           @arg @ref FL_RCC_PLL_PSC_DIV2
  *           @arg @ref FL_RCC_PLL_PSC_DIV4
  *           @arg @ref FL_RCC_PLL_PSC_DIV8
  *           @arg @ref FL_RCC_PLL_PSC_DIV12
  *           @arg @ref FL_RCC_PLL_PSC_DIV16
  *           @arg @ref FL_RCC_PLL_PSC_DIV24
  *           @arg @ref FL_RCC_PLL_PSC_DIV32

  */
void RCC_PLL_ConfigDomain_SYS(uint32_t Source, uint32_t PLL_R, uint32_t PLL_DB)
{
    MODIFY_REG(CMU->PLLCR, CMU_PLLCR_DB_Msk | CMU_PLLCR_REFPRSC_Msk  | CMU_PLLCR_INSEL_Msk,
               ((PLL_DB - 1) << CMU_PLLCR_DB_Pos) | PLL_R | Source);
}

void XTHFInit(const FL_CMU_XTHF_InitTypeDef *init)
{
    uint32_t counter = 0;
    uint32_t readystatus = 0;

    FL_GPIO_InitTypeDef      gpioInitStruction = { 0x0 };
    FL_NVIC_ConfigTypeDef    InterruptConfigStruct = { 0x0 };

    /* PC2,PC3 配置成模拟功能，外接XTHF */
    gpioInitStruction.pin           = FL_GPIO_PIN_2 | FL_GPIO_PIN_3;
    gpioInitStruction.mode          = FL_GPIO_MODE_ANALOG;
    gpioInitStruction.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    gpioInitStruction.pull          = FL_GPIO_BOTH_DISABLE;
    gpioInitStruction.remapPin      = FL_GPIO_PINREMAP_FUNCTON1;
    gpioInitStruction.analogSwitch  = FL_DISABLE;
    gpioInitStruction.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    (void)FL_GPIO_Init(GPIOC, &gpioInitStruction);

    FL_CMU_XTHF_WriteDriverStrength(init->DriverStrength);
    FL_CMU_XTHF_SetStartWaitTime(init->StartWaitTime);
    FL_CMU_XTHF_Enable();
    do
    {
        readystatus = FL_CMU_XTHF_IsReady();
        counter++;

        if((counter % 100) == 0)
        {
            FL_IWDT_ReloadCounter(IWDT);  
        }
    } while((readystatus != 0x1U) && (counter != XTHF_TIMEOUT));

    /*清除高频晶体停振检测标志*/
    FL_CMU_ClearFlag_XTHFFail();
    /*使能高频检测报警中断*/
    FL_CMU_EnableIT_XTHFFail();

    InterruptConfigStruct.preemptPriority = 0x00;
    /*系统时钟选择错误中断*/
    FL_NVIC_Init(&InterruptConfigStruct, FDET_IRQn);

}
/**
  * @brief  选择外部XTHF作为锁相环的参考时钟
  *
  * @param  clock PLL倍频输出频率
  * @note   输出时钟频率为1M*（clock+1），最高输出频率可达72M
  * @retval none
  */
uint32_t SelXTHFToPLL(const FL_CMU_XTHF_InitTypeDef *xthfInit,
                      const FL_CMU_PLL_InitTypeDef   *pllInit)
{
    FL_NVIC_ConfigTypeDef    InterruptConfigStruct;
    uint32_t counter = 0;
    uint32_t readystatus = 0;

    if(pllInit->Multiplier > 72)
    {
        /*超出PLL倍频最高范围*/
        return 1;
    }

    else
    {
        /*xthf配置*/
        XTHFInit(xthfInit);
        /*清除时钟选择错误标志*/
        FL_CMU_ClearFlag_SysclkSecleError();
        /*使能时钟选择错误中断*/
        FL_CMU_EnableIT_SYSCLKSelectError();

        InterruptConfigStruct.preemptPriority = 0x00;
        /*系统时钟选择错误中断*/
        FL_NVIC_Init(&InterruptConfigStruct, FDET_IRQn);

        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE);

        RCC_PLL_ConfigDomain_SYS(FL_CMU_PLL_CLK_SOURCE_XTHF, pllInit->Prescaler, pllInit->Multiplier);

        FL_CMU_PLL_Enable();  

        do
        {
            readystatus = FL_CMU_IsActiveFlag_PLLReady();
            counter++;

            if((counter % 100) == 0)
            {
                FL_IWDT_ReloadCounter(IWDT); 
            }
        } while((readystatus != 0x1U) && (counter != PLL_TIMEOUT)); 

        
        if(readystatus==0x1U)
        {
            FL_CMU_SetAHBPrescaler(FL_CMU_AHBCLK_PSC_DIV1);
            FL_CMU_SetAPBPrescaler(FL_CMU_APBCLK_PSC_DIV1);
            FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);

            if(pllInit->Multiplier <= 24)
            {
            /*读等待周期配置 0 wait cycle*/
                FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_0CYCLE); 
            }
            else if(pllInit->Multiplier <=48)   
            {
            /*读等待周期配置 1 wait cycle*/
                FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_1CYCLE); 
            }
            else
            {
                /*读等待周期配置 2 wait cycle*/
                FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE); 
            }
            SystemCoreClockUpdate();
            return 0;
        }
    }
    return 1;
}



void RCHFInit(uint32_t RCHFclock)
{
    switch(RCHFclock)
    {
        case FL_CMU_RCHF_FREQUENCY_8MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF8M_TRIM);
            break;
        case FL_CMU_RCHF_FREQUENCY_16MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF16M_TRIM);
            break;
        case FL_CMU_RCHF_FREQUENCY_24MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF24M_TRIM);
            break;
        case FL_CMU_RCHF_FREQUENCY_32MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF32M_TRIM);
            break;
        default:
            FL_CMU_RCHF_WriteTrimValue(RCHF8M_TRIM);
            break;
    }

    FL_CMU_RCHF_SetFrequency(RCHFclock);
    

}

