  .syntax unified
  .cpu cortex-m0plus
  .fpu softvfp
  .thumb

.global  Default_Handler

/* start address for the initialization values of the .data section.
defined in linker script */
.word  _sidata
/* start address for the .data section. defined in linker script */
.word  _sdata
/* end address for the .data section. defined in linker script */
.word  _edata
/* start address for the .bss section. defined in linker script */
.word  _sbss
/* end address for the .bss section. defined in linker script */
.word  _ebss

  .section  .text.Reset_Handler
  .weak  Reset_Handler
  .type  Reset_Handler, %function
Reset_Handler:  
   ldr   r0, =_estack
   mov   sp, r0          /* set stack pointer */

/* Copy the data segment initializers from flash to SRAM */
  movs  r1, #0
  b  LoopCopyDataInit

CopyDataInit:
  ldr  r3, =_sidata
  ldr  r3, [r3, r1]
  str  r3, [r0, r1]
  adds  r1, r1, #4

LoopCopyDataInit:
  ldr  r0, =_sdata
  ldr  r3, =_edata
  adds  r2, r0, r1
  cmp  r2, r3
  bcc  CopyDataInit
  ldr  r2, =_sbss
  b  LoopFillZerobss
/* Zero fill the bss segment. */
FillZerobss:
  movs  r3, #0
  str  r3, [r2]
  adds r2, r2, #4

LoopFillZerobss:
  ldr  r3, = _ebss
  cmp  r2, r3
  bcc  FillZerobss

/* Call the clock system intitialization function.*/
  bl  SystemInit
/* Call static constructors */
//    bl __libc_init_array
/* Call the application's entry point.*/
  bl  main

LoopForever:
  b LoopForever

  .size  Reset_Handler, .-Reset_Handler
  .section  .text.Default_Handler,"ax",%progbits
Default_Handler:
Infinite_Loop:
  b  Infinite_Loop
  .size  Default_Handler, .-Default_Handler

/******************************************************************************
*
* The minimal vector table for a Cortex M0.  Note that the proper constructs
* must be placed on this to ensure that it ends up at physical address
* 0x0000.0000.
*
******************************************************************************/
  .section  .isr_vector,"a",%progbits
  .global  g_pfnVectors
g_pfnVectors:
  .word  _estack
  .word  Reset_Handler
  .word  NMI_Handler
  .word  HardFault_Handler
  .word  0
  .word  0
  .word  0
  .word  0
  .word  0
  .word  0
  .word  0
  .word  SVC_Handler
  .word  0
  .word  0
  .word  PendSV_Handler
  .word  SysTick_Handler
  /* External Interrupts */
  .word  WDT_IRQHandler            /* 0:  IWDT &  WWDT      */
  .word  SVD_IRQHandler            /* 1:  SVD               */
  .word  RTC_IRQHandler            /* 2:  RTC               */
  .word  FLASH_IRQHandler          /* 3:  FLASH             */
  .word  FDET_IRQHandler           /* 4:  FDET              */
  .word  ADC_IRQHandler            /* 5:  ADC               */
  .word  RAMP_IRQHandler           /* 6:  RAMP              */
  .word  SPI0_IRQHandler           /* 7:  SPI0              */
  .word  SPI1_IRQHandler           /* 8:  SPI1              */
  .word  DAC_IRQHandler            /* 9:  DAC               */
  .word  UART0_IRQHandler          /* 10: UART0             */
  .word  UART1_IRQHandler          /* 11: UART1             */
  .word  UART2_IRQHandler          /* 12: UART2             */
  .word  UART3_IRQHandler          /* 13: UART3             */
  .word  UART4_IRQHandler          /* 14: UART4             */
  .word  FSCAN_IRQHandler          /* 15: FSCAN             */
  .word  LPUARTx_IRQHandler        /* 16: LPUART            */
  .word  I2CSMBx_IRQHandler        /* 17: I2Cx              */
  .word  CLM_PGL_IRQHandler        /* 18: CLM_PGL           */
  .word  AES_IRQHandler            /* 19: AES               */
  .word  LPTIM16_IRQHandler        /* 20: LPTIM16           */
  .word  DMA_IRQHandler            /* 21: DMA               */
  .word  WKUPx_IRQHandler          /* 22: WKUP              */
  .word  SENTx_IRQHandler          /* 23: SENT              */
  .word  BSTIM16_IRQHandler        /* 24: BSTIM16           */
  .word  COMPx_IRQHandler          /* 25: COMPx             */
  .word  GPTIM0_IRQHandler         /* 26: GPTIM0            */
  .word  GPTIM1_IRQHandler         /* 27: GPTIM1            */
  .word  ATIM_IRQHandler           /* 28: ATIM              */
  .word  TAU0_IRQHandler           /* 39: TAU0              */
  .word  GPIO_IRQHandler           /* 30: GPIO              */
  .word  LED_LCD_IRQHandler        /* 31: LED_LCD           */

/*******************************************************************************
*
* Provide weak aliases for each Exception handler to the Default_Handler.
* As they are weak aliases, any function with the same name will override
* this definition.
*
*******************************************************************************/
   .weak      NMI_Handler
   .thumb_set NMI_Handler,Default_Handler

   .weak      HardFault_Handler
   .thumb_set HardFault_Handler,Default_Handler

   .weak      SVC_Handler
   .thumb_set SVC_Handler,Default_Handler

   .weak      PendSV_Handler
   .thumb_set PendSV_Handler,Default_Handler

   .weak      SysTick_Handler
   .thumb_set SysTick_Handler,Default_Handler

   .weak      WDT_IRQHandler
   .thumb_set WDT_IRQHandler,Default_Handler

   .weak      SVD_IRQHandler
   .thumb_set SVD_IRQHandler,Default_Handler

   .weak      RTC_IRQHandler
   .thumb_set RTC_IRQHandler,Default_Handler

   .weak      FLASH_IRQHandler
   .thumb_set FLASH_IRQHandler,Default_Handler

   .weak      FDET_IRQHandler
   .thumb_set FDET_IRQHandler,Default_Handler

   .weak      ADC_IRQHandler
   .thumb_set ADC_IRQHandler,Default_Handler

   .weak      RAMP_IRQHandler
   .thumb_set RAMP_IRQHandler,Default_Handler

   .weak      SPI0_IRQHandler
   .thumb_set SPI0_IRQHandler,Default_Handler

   .weak      SPI1_IRQHandler
   .thumb_set SPI1_IRQHandler,Default_Handler

   .weak      DAC_IRQHandler
   .thumb_set DAC_IRQHandler,Default_Handler

   .weak      UART0_IRQHandler
   .thumb_set UART0_IRQHandler,Default_Handler

   .weak      UART1_IRQHandler
   .thumb_set UART1_IRQHandler,Default_Handler

   .weak      UART2_IRQHandler
   .thumb_set UART2_IRQHandler,Default_Handler

   .weak      UART3_IRQHandler
   .thumb_set UART3_IRQHandler,Default_Handler

   .weak      UART4_IRQHandler
   .thumb_set UART4_IRQHandler,Default_Handler

   .weak      FSCAN_IRQHandler
   .thumb_set FSCAN_IRQHandler,Default_Handler

   .weak      LPUARTx_IRQHandler
   .thumb_set LPUARTx_IRQHandler,Default_Handler

   .weak      I2CSMBx_IRQHandler
   .thumb_set I2CSMBx_IRQHandler,Default_Handler

   .weak      CLM_PGL_IRQHandler
   .thumb_set CLM_PGL_IRQHandler,Default_Handler

   .weak      AES_IRQHandler
   .thumb_set AES_IRQHandler,Default_Handler

   .weak      LPTIM16_IRQHandler
   .thumb_set LPTIM16_IRQHandler,Default_Handler

   .weak      DMA_IRQHandler
   .thumb_set DMA_IRQHandler,Default_Handler

   .weak      WKUPx_IRQHandler
   .thumb_set WKUPx_IRQHandler,Default_Handler

   .weak      SENTx_IRQHandler
   .thumb_set SENTx_IRQHandler,Default_Handler

   .weak      BSTIM16_IRQHandler
   .thumb_set BSTIM16_IRQHandler,Default_Handler

   .weak      COMPx_IRQHandler
   .thumb_set COMPx_IRQHandler,Default_Handler

   .weak      GPTIM0_IRQHandler
   .thumb_set GPTIM0_IRQHandler,Default_Handler

   .weak      GPTIM1_IRQHandler
   .thumb_set GPTIM1_IRQHandler,Default_Handler
   
   .weak      ATIM_IRQHandler
   .thumb_set ATIM_IRQHandler,Default_Handler

   .weak      TAU0_IRQHandler
   .thumb_set TAU0_IRQHandler,Default_Handler
   
   .weak      GPIO_IRQHandler
   .thumb_set GPIO_IRQHandler,Default_Handler

   .weak      LED_LCD_IRQHandler
   .thumb_set LED_LCD_IRQHandler,Default_Handler

